package cd.casic.module.machine.service.impl;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.framework.commons.util.object.BeanUtils;
import cd.casic.module.machine.controller.vo.SecretKeyVO;
import cd.casic.module.machine.dal.dataobject.SecretKeyDO;
import cd.casic.module.machine.dal.mysql.SecretKeyMapper;
import cd.casic.module.machine.service.MachineInfoService;
import cn.hutool.core.io.resource.ResourceUtil;
import cn.hutool.core.util.IdUtil;
import cd.casic.module.machine.utils.AliYunOssClient;
import cd.casic.module.machine.service.SecretKeyService;
import com.google.common.annotations.VisibleForTesting;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import static cd.casic.framework.commons.exception.util.ServiceExceptionUtil.exception;
import static cd.casic.module.machine.contants.MachineErrorCodeConstants.*;

/**
 * 密钥服务实现类
 */
@Service("secretKeyService")
public class SecretKeyServiceImpl implements SecretKeyService {

    @Resource
    private MachineInfoService machineInfoService;

    @Resource
    private AliYunOssClient aliYunOssClient;

    @Resource
    private SecretKeyMapper secretKeyMapper;



    @Override
    public SecretKeyVO getSecretKey(Long id){
        SecretKeyDO secretKeyDO = validateSecretKeyExists(id);
        return BeanUtils.toBean(secretKeyDO, SecretKeyVO.class);
    }

    @Override
    public Long createSecretKey(SecretKeyVO secretKeyVO){
        validateSecretKeyAdd(secretKeyVO);
        String ossPath = upLoadSecretKey(secretKeyVO.getPath());
        //检查得到的oss路径是否为空
        validateSecretKeyPath(ossPath);
        secretKeyVO.setPath(ossPath);
        SecretKeyDO secretKeyDO = BeanUtils.toBean(secretKeyVO, SecretKeyDO.class);
        //todo检查密钥合法

        secretKeyMapper.insert(secretKeyDO);
        return secretKeyDO.getId();


    }
    @Override
    public void updateSecretKey(SecretKeyVO secretKeyVO) {
        SecretKeyDO secretKeyDO = validateSecretKeyExists(secretKeyVO.getId());
        //如果路径改变==改变密钥
        if (!secretKeyDO.getPath().equals(secretKeyVO.getPath())) {
            //todo检查密钥合法
            String ossPath = upLoadSecretKey(secretKeyVO.getPath());
            BeanUtils.copyProperties(secretKeyVO, secretKeyDO);
            secretKeyDO.setPath(ossPath);
        }
        else {
            BeanUtils.copyProperties(secretKeyVO,secretKeyDO);
        }
        secretKeyMapper.updateById(secretKeyDO);
    }

    @Override
    public void bindingMachine(Long id,List<Long> machineInfoIds) {
        validateSecretKeyExists(id);
        machineInfoService.bindingSecretKey(machineInfoIds, id);
    }

    @Override
    @Transactional
    public void deleteSecretKeyList(List<Long> ids) {
        ids.forEach(
                secretKeyId -> {
                    SecretKeyDO secretKeyDO = validateSecretKeyExists(secretKeyId);
                    if (secretKeyDO.getPath() != null && !secretKeyDO.getPath().isEmpty()){
                        try {
                            //文件名
                            //删除子目录文件，需要在前面加上根目录文件路径
                            String fileName = secretKeyDO.getPath().substring(secretKeyDO.getPath().lastIndexOf("/") + 1);
                            aliYunOssClient.delete(fileName);
                        } catch (Exception e) {
                            throw exception(DELETE_FILE_FAIL);
                        }
                    }
                }
        );

        //绑定的机器全部设置为空
        machineInfoService.unBindingSecretKey(ids);

        secretKeyMapper.deleteBatchIds(ids);
    }

    @Override
    public PageResult<SecretKeyDO> getSecretKeypage(SecretKeyVO secretKeyVO) {
        return secretKeyMapper.selectPage(secretKeyVO);
    }

    public String upLoadSecretKey(String localPath) {
        //使用S3FileClient上传文件
        aliYunOssClient.init();
        //传输到指定文件，需要在path前面加上文件路径
        String path = IdUtil.fastSimpleUUID() + ".txt";
        //上传文件是从本地上传，这里传的是本地文件地址
        byte[] content = ResourceUtil.readBytes(localPath);
        String ossPath;
        try {
            ossPath = aliYunOssClient.upload(content, path, "txt");
        }catch (Exception e) {
            throw exception(UPLOADING_FILE_FAIL);
        }
        return ossPath;
    }

    @VisibleForTesting
    void validateSecretKeyAdd(SecretKeyVO secretKeyVO) {
        if (secretKeyVO==null) {
            throw exception(SECRET_KEY_NULL);
        }
        if (secretKeyVO.getPath().isEmpty()) {
            throw exception(SECRET_KEY_PATH_NULL);
        }
    }

    @VisibleForTesting
    void validateSecretKeyPath(String path) {
        if (path.isEmpty()) {
            throw exception(SECRET_KEY_PATH_NULL);
        }
    }


    @VisibleForTesting
    SecretKeyDO validateSecretKeyExists(Long id) {
        if (id == null) {
            return null;
        }
        SecretKeyDO secretKeyDO = secretKeyMapper.selectById(id);
        if (secretKeyDO == null) {
            throw exception(SECRET_KEY_NOT_EXISTS);
        }
        return secretKeyDO;
    }

}
